@@ -1,59 +1,4 @@
-#' Individualized cost-effectiveness analysis
-#'
-#' These functions are deprecated, use [cea()] and [cea_pw()] instead. 
-#' @param x An object of simulation output characterizing the probability distribution
-#' of clinical effectiveness and costs.?ic
-#' @param ... Further arguments passed to or from other methods. 
-#' @export
-#' @rdname icea
-icea <- function(x, ...) {
-  .Deprecated("cea")
-  UseMethod("cea")
-}
-
-#' @export
-#' @rdname icea
-icea_pw <- function(x, ...) {
-  .Deprecated("cea_pw")
-  UseMethod("cea_pw")
-}
-
-#'  A cost-effectiveness object
-#'
-#' An object that summarizes simulated measures of clinical effectiveness and costs from a simulation model for use in a cost-effectiveness analysis.
-#'
-#' 
-#' @format 
-#' A list containing two elements:
-#' \itemize{
-#' \item{costs}{ Total (discounted) costs by category.}
-#' \item{qalys}{ (Discounted) quality-adjusted life-years.}
-#' }
-#' 
-#' @section Costs:
-#' The 'costs' \code{\link{data.table}} contains the following columns:
-#' \describe{
-#' \item{category}{The cost category.}
-#' \item{dr}{The discount rate.}
-#' \item{sample}{A randomly sampled parameter set from the probabilistic sensitivity analysis (PSA)}
-#' \item{strategy_id}{The treatment strategy ID.}
-#' \item{grp}{An optional column denoting a subgroup. If not included, it is assumed that a single subgroup is being analyzed.}
-#' \item{costs}{Costs.}
-#' }
-#' 
-#' @section Quality-adjusted life-years:
-#' The 'qalys' \code{\link{data.table}} contains the following columns:
-#' \describe{
-#' \item{dr}{The discount rate.}
-#' \item{sample}{A randomly sampled parameter set from the probabilistic sensitivity analysis (PSA)}
-#' \item{strategy_id}{The treatment strategy ID.}
-#' \item{grp}{An optional column denoting a subgroup. If not included, it is assumed that a single subgroup is being analyzed.}
-#' \item{qalys}{Quality-adjusted life-years}
-#' }
-#' 
-#' @name ce
-NULL
-
+# Cost-effectiveness analysis --------------------------------------------------
 #' Cost-effectiveness analysis
 #'
 #' Conduct cost-effectiveness analysis (CEA) given output of an economic
@@ -106,7 +51,7 @@ NULL
 #' \code{cea_pw} also returns a list of four `data.table` elements:
 #'  \describe{
 #'   \item{summary}{A data.table of the mean, 2.5% quantile, and 97.5% 
-#'   quantile by strategy and group for clinical effectiveness and costs.}
+#'   quantile by strategy and group for incremental clinical effectiveness and costs.}
 #'   \item{delta}{Incremental effectiveness and incremental cost for each simulated
 #'   parameter set by strategy and group. Can be used to plot a cost-effectiveness plane. }
 #'   \item{ceac}{Values needed to plot a cost-effectiveness acceptability curve by
@@ -117,36 +62,54 @@ NULL
 #' }
 #' @name cea
 #' @examples
-#' # simulation output
-#' n_samples <- 100
-#' sim <- data.frame(sample = rep(seq(n_samples), 4),
+#' library("data.table")
+#' library("ggplot2")
+#' theme_set(theme_bw())
+#' 
+#' # Simulation output
+#' n_samples <- 30
+#' 
+#' sim <- data.table(sample = rep(seq(n_samples), 4),
 #'                   c = c(rlnorm(n_samples, 5, .1), rlnorm(n_samples, 5, .1),
 #'                         rlnorm(n_samples, 11, .1), rlnorm(n_samples, 11, .1)),
 #'                   e = c(rnorm(n_samples, 8, .2), rnorm(n_samples, 8.5, .1),
 #'                         rnorm(n_samples, 11, .6), rnorm(n_samples, 11.5, .6)),
-#'                   strategy = rep(paste0("Strategy ", seq(1, 2)),
-#'                                  each = n_samples * 2),
-#'                   grp = rep(rep(c("Group 1", "Group 2"),
-#'                             each = n_samples), 2)
+#'                   strategy_id = rep(1:2, each = n_samples * 2),
+#'                   grp_id = rep(rep(1:2, each = n_samples), 2)
 #')
 #'
-#' # cea
-#' cea <- cea(sim, k = seq(0, 200000, 500), sample = "sample", strategy = "strategy",
-#'              grp = "grp", e = "e", c = "c")
-#' names(cea)
-#' # The probability that each strategy is the most cost-effective 
-#' # in each group with a willingness to pay of 20,000
-#' library("data.table")
-#' cea$mce[k == 20000]
+#' # Cost-effectiveness analysis
+#' cea_out <- cea(sim, k = seq(0, 200000, 500), sample = "sample", 
+#'                strategy = "strategy_id", grp = "grp_id", 
+#'                e = "e", c = "c")
+#' names(cea_out)
 #' 
-#' # cea_pw
-#' cea_pw <-  cea_pw(sim,  k = seq(0, 200000, 500), comparator = "Strategy 1",
-#'                     sample = "sample", strategy = "strategy", grp = "grp",
-#'                      e = "e", c = "c")
-#' names(cea_pw)
-#' # cost-effectiveness acceptability curve
-#' head(cea_pw$ceac[k >= 20000])
-#' icer_tbl(cea_pw)
+#' ## Some sample output
+#' ## The probability that each strategy is the most cost-effective 
+#' ## in each group with a willingness to pay of 20,000
+#' cea_out$mce[k == 20000]
+#' 
+#' # Pairwise cost-effectiveness analysis
+#' cea_pw_out <-  cea_pw(sim,  k = seq(0, 200000, 500), comparator = 1,
+#'                       sample = "sample", strategy = "strategy_id", 
+#'                       grp = "grp_id", e = "e", c = "c")
+#' names(cea_pw_out)
+#' 
+#' ## Some sample output
+#' ## The cost-effectiveness acceptability curve
+#' head(cea_pw_out$ceac[k >= 20000])
+#' 
+#' # Summarize the incremental cost-effectiveness ratio
+#' labs <- list(strategy_id = c("Strategy 1" = 1, "Strategy 2" = 2),
+#'              grp_id = c("Group 1" = 1, "Group 2" = 2))
+#' format(icer(cea_pw_out, labels = labs))
+#' 
+#' # Plots
+#' plot_ceplane(cea_pw_out, label = labs)
+#' plot_ceac(cea_out, label = labs)
+#' plot_ceac(cea_pw_out, label = labs)
+#' plot_ceaf(cea_out, label = labs)
+#' plot_evpi(cea_out, label = labs)
 #' @export
 cea <- function(x, ...) {
   UseMethod("cea")
@@ -369,177 +332,158 @@ cea_table <- function(x, strategy, grp, e, c, icer = FALSE){
     ie_mean <- paste0(e, "_mean")
     ic_mean <- paste0(c, "_mean")
     ret$icer <- ret[, ic_mean, with = FALSE]/ret[, ie_mean, with = FALSE]
-    ret$icer <- ifelse(ret[, ic_mean, with = FALSE] < 0 & ret[, ie_mean, with = FALSE] >= 0, "Dominates",
-                       ret$icer)
-    ret$icer <- ifelse(ret[, ic_mean, with = FALSE] > 0 & ret[, ie_mean, with = FALSE] <= 0, "Dominated",
-                       ret$icer)
   }
   return(ret)
 }
 
-format_costs <- function(x, digits){
-  formatC(x, format = "f", digits = digits, big.mark = ",")
-}
-
-format_qalys <- function(x, digits){
-  formatC(x, format = "f", digits = digits)
-}
-
-format_cri <- function(est, lower, upper, costs = TRUE, digits){
-  if (costs){
-    lower <- format_costs(lower, digits = digits)
-    upper <- format_costs(upper, digits = digits)
-  } else{
-    lower <- format_qalys(lower, digits = digits)
-    upper <- format_qalys(upper, digits = digits)
-  }
-  paste0(est, " (",lower, ", ", upper, ")")
-}
-
-#' ICER table
+# Incremental cost-effectiveness ratio------------------------------------------
+#' Incremental cost-effectiveness ratio
 #'
-#' Generate a table of incremental cost-effectiveness ratios given output from 
-#' [cea_pw()].
+#' Generate a tidy table of incremental cost-effectiveness ratios (ICERs) given output from 
+#' [cea_pw()] with `icer()` and format for pretty printing with `format.icer()`.
 #'
+#' @inheritParams set_labels
 #' @param x An object of class `cea_pw` returned by [cea_pw()].
-#' @param k Willingness to pay.
-#' @param cri If `TRUE`, credible intervals are computed; otherwise 
-#' they are not.
-#' @param prob A numeric scalar in the interval `(0,1)` giving the credible interval.
-#' Default is 0.95 for a 95 percent credible interval. 
-#' @param digits_qalys Number of digits to use to report QALYs.
-#' @param digits_costs Number of digits to use to report costs.
-#' @param output Should output be a `data.table` or a list of matrices for
-#' each group.
-#' @param rownames Row names for matrices when `output = "matrix"`.
-#' @param colnames Column names for matrices when `output = "matrix"`.
-#' @param drop If `TRUE`, then the result is coerced to the lowest possible dimension. 
-#' Relevant if `output = "matrix"` and there is one group, in which case a single
-#' matrix will be returned if `drop = TRUE` and a list of length 1 will be returned
-#' if `drop = FALSE`.
-#' @seealso [cea_pw()]
-#' @return If `output = "matrix"`, then a list of matrices (or a matrix if
-#' \code{drop = TRUE}) reporting incremental cost-effectiveness ratios (ICERs)
-#' by group. Specifically, each matrix contains five rows for: (i) 
-#' incremental quality-adjusted life-years (QALYs), (ii) incremental costs,
-#' (iii) the incremental net monetary benefit (NMB), (iv) the ICER, 
-#' and (v) a conclusion stating whether each strategy is cost-effective relative
-#'  to a comparator. The number of columns is equal to the
-#' number of strategies (including the comparator).
+#' @param prob A numeric scalar in the interval `(0,1)` giving the confidence interval.
+#' Default is 0.95 for a 95 percent interval. 
+#' @param k Willingness to pay per quality-adjusted life-year.
+#' @param ... Further arguments passed to and from methods. Currently unused. 
 #' 
-#' If `output = "data.table"`, then the results are reported as a `data.table`,
-#' with one row for each strategy and group combination.
+#' @details Note that `icer()` will report negative ICERs; however, `format()` will
+#' correctly note whether a treatment strategy is dominated by or dominates the 
+#' reference treatment. 
+#' 
+#' @return `icer()` returns an object of class `icer` that is a tidy 
+#' `data.table` with the following columns:
+#' \describe{
+#' \item{strategy}{The treatment strategy.}
+#' \item{grp}{The subgroup.}
+#' \item{outcome}{The outcome metric.}
+#' \item{estimate}{The point estimate computed as the average across the PSA samples.}
+#' \item{lower}{The lower limit of the confidence interval.}
+#' \item{upper}{The upper limit of the confidence interval.}
+#' }
+#' 
+#' `format.icer()` formats the table according to the arguments passed.
+#' 
+#' @seealso [`cea_pw()`]
 #' @export
-icer_tbl <- function(x, k = 50000, cri = TRUE, prob = 0.95, 
-                     digits_qalys = 2, 
-                     digits_costs = 0, output = c("matrix", "data.table"),
-                     rownames = NULL, colnames = NULL,
-                     drop = TRUE){
+icer <- function(x, prob = .95, k = 50000, labels = NULL, ...){
+  ie <- ic <- imbm <- inmb_lower <- inmb_mean <- inmb_upper <- 
+    ic_mean <- ie_mean <- outcome <- NULL
+  
   if (!inherits(x, "cea_pw")){
     stop("'x' must be an object of class 'cea_pw'",
          call. = FALSE)
   }
-  if (prob > 1 | prob < 0){
-    stop("'prob' must be in the interval (0,1)",
-         call. = FALSE)
-  }
-  
+  alpha <- ci_alpha(prob)
   strategy <- attributes(x)$strategy
-  grp <- attributes(x)$grp
-  output <- match.arg(output)
+  grp <- attributes(x)$grp 
+  
+  
+  # Estimates for costs, QALYs, and the ICER have already been computed
   tbl <- copy(x$summary)
-  tbl[, "icer" := get("ic_mean")/get("ie_mean")]  
-  tbl[, "inmb_numeric" := k * get("ie_mean") - get("ic_mean")]
   
-  # Formatting
-  tbl[, "iqalys" := format_qalys(get("ie_mean"), digits = digits_qalys)]
-  tbl[, "icosts" := format_costs(get("ic_mean"), digits = digits_costs)]
-  tbl[, "icer" := format_costs(get("icer"), digits = digits_costs)]
-  tbl[, "icer" := ifelse(get("ic_mean") < 0 & get("ie_mean") >= 0, "Dominates", get("icer"))]
-  tbl[, "icer" := ifelse(get("ic_mean") > 0 & get("ie_mean") <= 0, "Dominated", get("icer"))]
-  tbl[, "inmb" := format_costs(get("inmb_numeric"), digits = digits_costs)]
+  # Compute INMB given value of k passed to function
+  delta <- copy(x$delta)
+  delta[, inmb := k * ie - ic]
+  inmb <- delta[, list(inmb_lower = stats::quantile(inmb, alpha$lower),
+                       inmb_mean = mean(inmb),
+                       inmb_upper = stats::quantile(inmb, alpha$upper)),
+                by = c(strategy, grp)]
+  tbl <- cbind(tbl, 
+               inmb[, c("inmb_lower", "inmb_mean", "inmb_upper"), with = FALSE])
+  tbl[, icer_plane := fcase(
+    inmb_mean >= 0 & ic_mean > 0 & ie_mean > 0, "Cost-effective",
+    inmb_mean < 0 & ic_mean > 0 & ie_mean > 0, "Not cost-effective",
+    inmb_mean >= 0 & ic_mean < 0 & ie_mean < 0, "Cost-effective",
+    inmb_mean < 0 & ic_mean < 0 & ie_mean < 0, "Not cost-effective",
+    ic_mean < 0 & ie_mean >= 0, "Dominates",
+    ic_mean > 0 & ie_mean <= 0, "Dominated"
+  )]
+  icer_plane <- tbl$icer_plane
+  tbl <- melt(tbl, id.vars = c(strategy, grp),
+              measure.vars = list(c("ie_mean", "ic_mean", "inmb_mean", "icer"),
+                                  c("ie_lower", "ic_lower", "inmb_lower"),
+                                  c("ie_upper", "ic_upper", "inmb_upper")),
+              variable.factor = FALSE,
+              variable.name = "outcome",
+       value.name = c("estimate", "lower", "upper"))
+  tbl[, outcome := factor(outcome, levels = 1:4,
+                           labels = c("Incremental QALYs",
+                                      "Incremental costs",
+                                      "Incremental NMB",
+                                      "ICER"))]
   
-  if(cri){
-    prob_lower <- (1 - prob)/2
-    prob_upper <- 1 - prob_lower
-    x$delta[, "inmb" := k * get("ie") - get("ic")]
-    if (prob == 0.95){
-      tbl[, "iqalys" := format_cri(get("iqalys"), get("ie_lower"), get("ie_upper"), 
-                                   costs = FALSE,
-                                  digits = digits_qalys)]
-      tbl[, "icosts" := format_cri(get("icosts"), get("ic_lower"), get("ic_upper"),
-                                 costs = TRUE,
-                                  digits = digits_costs)]
-      inmb_dt <- x$delta[, list(mean = mean(get("inmb")),
-                          lower = stats::quantile(get("inmb"), prob_lower),
-                          upper = stats::quantile(get("inmb"), prob_upper)),
-                      by = c(strategy, grp)]
-      tbl[, "inmb" := format_cri(get("inmb"), inmb_dt$lower, inmb_dt$upper,
-                                 costs = TRUE,
-                                digits = digits_costs)]
-    } else {
-      cri_dt <- x$delta[, list(iqalys_lower = stats::quantile(get("ie"), prob_lower),
-                               iqalys_upper = stats::quantile(get("ie"), prob_upper),
-                               icosts_lower = stats::quantile(get("ic"), prob_lower),
-                               icosts_upper = stats::quantile(get("ic"), prob_upper),
-                               inmb_lower = stats::quantile(get("inmb"), prob_lower),
-                               inmb_upper = stats::quantile(get("inmb"), prob_upper)),
-                      by = c(strategy, grp)]
-      tbl[, "iqalys" := format_cri(get("iqalys"), cri_dt$iqalys_lower, 
-                                 cri_dt$iqalys_upper, costs = FALSE,
-                                  digits = digits_qalys)]
-      tbl[, "icosts" := format_cri(get("icosts"), cri_dt$icosts_lower, 
-                                 cri_dt$icosts_upper, costs = TRUE,
-                                  digits = digits_costs)]      
-      tbl[, "inmb" := format_cri(get("inmb"), cri_dt$inmb_lower, 
-                                 cri_dt$inmb_upper, costs = TRUE,
-                                digits = digits_costs)]       
-    }
-    x$delta[, "inmb" := NULL]
-  } # end credible interval calculations
-  tbl[, "conclusion" := ifelse(get("inmb_numeric") >= 0,
-                            "Cost-effective", "Not cost-effective")]
-  tbl <- tbl[, c(strategy, grp, "iqalys", "icosts", "inmb", "icer", "conclusion"),
-             with = FALSE]
-  
-  if (output == "matrix"){
-    tbl_list <- split(tbl, by = grp)
-    mat_list <- vector(mode = "list", length = length(tbl_list))
-    names(mat_list) <- names(tbl_list)
-    n_strategies <- length(unique(tbl[[strategy]]))
-    mat <- matrix(NA, nrow = 5, ncol = n_strategies + 1)
-    if(is.null(rownames)){
-      rownames(mat) <- c("Incremental QALYs", "Incremental costs", 
-                          "Incremental NMB", "ICER", "Conclusion")
-    } else{
-      rownames(mat) <- rownames
-    }
-    comp_pos <- attributes(x)$comparator_pos
-    if (is.null(colnames)){
-      strategy_names <- rep(NA, ncol(mat))
-      strategy_names[comp_pos] <- attributes(x)$comparator
-      strategy_names[-comp_pos] <- as.character(tbl_list[[1]][[strategy]])
-      colnames(mat) <- strategy_names
-    } else{
-      colnames(mat) <- colnames
-    }
-    for (i in 1:length(mat_list)){
-      mat[1, -comp_pos] <- tbl_list[[i]]$iqalys
-      mat[2, -comp_pos] <- tbl_list[[i]]$icosts
-      mat[3, -comp_pos] <- tbl_list[[i]]$inmb
-      mat[4, -comp_pos] <- tbl_list[[i]]$icer
-      mat[5, -comp_pos] <- tbl_list[[i]]$conclusion
-      mat[, comp_pos] <- "-"
-      mat_list[[i]] <- mat 
-    }
-    if (drop){
-      if(length(mat_list) == 1){
-        mat_list <- mat_list[[1]]
-      }
-    }
-    return(mat_list)
-  } else{
-    return(tbl)
-  }
+  # Values of treatment strategies and subgroups
+  set_labels(tbl, labels = labels)
+  setnames(tbl, c(strategy, grp), c("strategy", "grp"))
+
+  # Return
+  setorderv(tbl, c("grp", "strategy"))
+  setattr(tbl, "class", c("icer", "data.table", "data.frame"))
+  setattr(tbl, "k", k)
+  setattr(tbl, "icer_plane", icer_plane)
+  return(tbl[, ])
 }
 
+#' @rdname icer
+#' @param digits_qalys Number of digits to use to report QALYs.
+#' @param digits_costs Number of digits to use to report costs.
+#' @param pivot_from Character vector denoting a column or columns used to 
+#' "widen" the data. Should either be `"strategy"`, `"grp"`, `"outcome"`,
+#' or some combination of the three. There will be one column for each value of 
+#' the variables in `pivot_from`. Default is to widen so there is a column for each treatment
+#' strategy. Set to `NULL` if you do not want to widen the table. 
+#' @param drop_grp If `TRUE`, then the group column will be removed if there is only
+#' one subgroup; other it will be kept. If `FALSE`, then the `grp` column is never
+#' removed. 
+#' @param pretty_names Logical. If `TRUE`, then the columns `strategy`, `grp`,
+#' `outcome`, and `value` are renamed (if they exist) to `Strategy`, 
+#' `Group`, `Outcome`, and `Value`.
+#' @export
+format.icer <- function(x, digits_qalys = 2, digits_costs = 0,
+                        pivot_from = "strategy", drop_grp = TRUE,
+                        pretty_names = TRUE,...) {
+  value <- outcome <- estimate <- lower <- upper <- grp <- NULL 
+  y <- copy(x)
+  
+  # Format values
+  icer_plane <- attr(x, "icer_plane")
+  icer_plane <- rep(icer_plane, each = nrow(y)/length(icer_plane))
+  y[, value := fcase(
+    outcome %in% c("Incremental costs", "Incremental NMB"),
+      format_ci(estimate, lower, upper, costs = TRUE,
+                digits = digits_costs),
+    outcome == "Incremental QALYs", 
+      format_ci(estimate, lower, upper, costs = FALSE,
+                digits = digits_qalys),
+    outcome == "ICER" & icer_plane == "Dominates", 
+      "Dominates",
+    outcome == "ICER" & icer_plane == "Dominated", 
+      "Dominated",
+    outcome == "ICER" & !icer_plane %in% c("Dominates", "Dominated"), 
+      format_costs(estimate, digits = digits_costs)
+  )]
+  y[, c("estimate", "lower", "upper") := NULL]
+  
+  # Potentially pivot wider and drop groups
+  id <- c("grp", "strategy", "outcome")
+  y[, outcome := factor(outcome, levels = unique(y$outcome))]
+  y <- format_summary_default(y, pivot_from = pivot_from,
+                              id_cols = id,
+                              drop_grp = drop_grp)
+  
+  # Pretty names
+  if (pretty_names) {
+    setnames(
+      y, 
+      c("grp", "strategy", "outcome", "value"),
+      c("Group", "Strategy", "Outcome", "Value"),
+      skip_absent = TRUE
+    )
+  } 
+  
+  # Return
+  return(y[, ])
+}
